@@ -1,38 +1,19 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /**
  * ms-ole.c: MS Office OLE support for Gnumeric
  *
  * Authors:
  *    Michael Meeks (michael@ximian.com)
  *    Arturo Tena   (arturo@directmail.org)
+ *    Dom Lachowicz (doml@appligent.com)
  *
  * Copyright 1998-2000 Helix Code, Inc., Arturo Tena
  **/
 
-#include <stdio.h>
-
-/* BSDs require unistd.h before including stat.h */
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <sys/stat.h>	/* for struct stat */
-#include <fcntl.h>
-
-#include <assert.h>
-#include <ctype.h>
-#include <glib.h>
-#include <string.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include <libole2-config.h>
 #include <libole2/ms-ole.h>
 
-#ifdef HAVE_MMAP
-#include <sys/mman.h>
-#endif
-
+/* BSDs require unistd.h before including stat.h */
 #ifdef HAVE_UNISTD_H
 #	include <unistd.h>
 #else
@@ -45,6 +26,16 @@
 #	define O_NONBLOCK 0x4000
 #endif
 
+#include <sys/stat.h>	/* for struct stat */
+#include <fcntl.h>
+#include <stdio.h>
+#include <assert.h>
+#include <ctype.h>
+#include <string.h>
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif
+
 #ifndef PROT_READ
 #define PROT_READ 0x1
 #endif
@@ -53,7 +44,7 @@
 #define PROT_WRITE 0x2
 #endif
 
-#ifndef MAP_FAILED
+#if !defined(MAP_FAILED) || defined(__osf__)
 /* Someone needs their head examining - BSD ? */
 #	define MAP_FAILED ((void *)-1)
 #endif
@@ -108,7 +99,7 @@ struct _MsOle {
 	MsOleSysWrappers *syswrap;
 
 	char              mode;
-	int               file_des;
+	MsOleHandleType   file_des;
 	int               dirty;
 	GArray           *bb;      /* Big  blocks status  */
 	GArray           *sb;      /* Small block status  */
@@ -120,95 +111,100 @@ struct _MsOle {
 /* end if memory mapped */
 };
 
-
 #define BLOCK_COUNT(f) (((f)->length + BB_BLOCK_SIZE - 1) / BB_BLOCK_SIZE)
 
-
-/**
- * Default system calls wrappers
- **/
-
-static int
-open2_wrap (const char *pathname, int flags)
+static MsOleHandleType
+open2_wrap (const char *pathname, int flags, gpointer closure)
 {
+	gint fd;
 #ifdef O_BINARY
-	return open (pathname, flags | O_BINARY);
+	fd = open (pathname, flags | O_BINARY);
 #else
-	return open (pathname, flags);
+	fd = open (pathname, flags);
 #endif
+
+	if (fd == -1)
+		return BAD_MSOLE_HANDLE;
+	return GINT_TO_POINTER(fd);
 }
 
-static int
-open3_wrap (const char *pathname, int flags, mode_t mode)
+static MsOleHandleType
+open3_wrap (const char *pathname, int flags, mode_t mode, gpointer closure)
 {
+	gint fd;
 #ifdef O_BINARY
-	return open (pathname, flags | O_BINARY, mode);
+	fd = open (pathname, flags | O_BINARY, mode);
 #else
-	return open (pathname, flags, mode);
+	fd = open (pathname, flags, mode);
 #endif
+
+	if (fd == -1)
+		return BAD_MSOLE_HANDLE;
+	return GINT_TO_POINTER(fd);
 }
 
 static ssize_t
-read_wrap (int fd, void *buf, size_t count)
+read_wrap (MsOleHandleType fd, void *buf, size_t count, gpointer closure)
 {
-	return read (fd, buf, count);
+	return read (GPOINTER_TO_INT(fd), buf, count);
 }
 
 static int
-close_wrap (int fd)
+close_wrap (MsOleHandleType fd, gpointer closure)
 {
-	return close (fd);
+	return close (GPOINTER_TO_INT(fd));
 }
 
 static ssize_t
-write_wrap (int fd, const void *buf, size_t count)
+write_wrap (MsOleHandleType fd, const void *buf, size_t count,
+	    gpointer closure)
 {
-	return write (fd, (void *)buf, count);
+	return write (GPOINTER_TO_INT(fd), (void *)buf, count);
 }
 
 static off_t
-lseek_wrap (int fd, off_t offset, int whence)
+lseek_wrap (MsOleHandleType fd, off_t offset, int whence, gpointer closure)
 {
-	return lseek (fd, offset, whence);
+	return lseek (GPOINTER_TO_INT(fd), offset, whence);
 }
 
 static int
-isregfile_wrap (int fd)
+isregfile_wrap (MsOleHandleType fd, gpointer closure)
 {
 	struct stat st;
 
-	if (fstat (fd, &st))
+	if (fstat (GPOINTER_TO_INT(fd), &st))
 		return 0;
 
 	return S_ISREG (st.st_mode);
 }
 
 static int
-getfilesize_wrap (int fd, guint32 *size)
+getfilesize_wrap (MsOleHandleType fd, guint32 *size, gpointer closure)
 {
 	struct stat st;
 
-	if (fstat (fd, &st))
+	if (fstat (GPOINTER_TO_INT(fd), &st))
 		return -1;
 
 	*size = st.st_size;
 	return 0;
 }
 
-#ifdef HAVE_MMAP
+#if defined(HAVE_MMAP)
 static void *
 mmap_wrap (void *start, size_t length, int prot,
-	   int flags, int fd, off_t offset)
+	   int flags, MsOleHandleType fd, off_t offset, gpointer closure)
 {
-	return mmap (start, length, prot, flags, fd, offset);
+	return mmap (start, length, prot, flags, GPOINTER_TO_INT(fd), offset);
 }
 
 static int
-munmap_wrap (void *start, size_t length)
+munmap_wrap (void *start, size_t length, gpointer closure)
 {
 	return munmap (start, length);
 }
-#endif
+#endif /* HAVE_MMAP */
 
 static MsOleSysWrappers ms_ole_default_wrappers = {
 	open2_wrap,
@@ -220,24 +216,92 @@ static MsOleSysWrappers ms_ole_default_wrappers = {
 	isregfile_wrap,
 	getfilesize_wrap,
 
-#ifdef HAVE_MMAP
+#if defined(HAVE_MMAP)
 	mmap_wrap,
-	munmap_wrap
+	munmap_wrap,
 #else
 	NULL,
-	NULL
+	NULL,
 #endif
+	NULL /* no closure */
 };
 
+/*
+ * The default filesystem wrappers
+ */
+static MsOleSysWrappers * default_wrappers = &ms_ole_default_wrappers;
+
 static void
 take_wrapper_functions (MsOle *f, MsOleSysWrappers *wrappers)
 {
 	if (wrappers == NULL)
-		f->syswrap = &ms_ole_default_wrappers;
+		f->syswrap = default_wrappers;
 	else
 		f->syswrap = wrappers;
 }
 
+/*
+ * Initialize MS-OLE
+ * @wrappers - if NULL, use the default FILE* based filesystem
+ * else use the supplied filesystem
+ */
+void
+ms_ole_init (MsOleSysWrappers *wrappers)
+{
+	static gboolean bInitted = FALSE;
+	g_return_if_fail(!bInitted);
+
+	if (wrappers != NULL)
+		default_wrappers = wrappers;
+	bInitted = TRUE;
+}
+
+/*
+ * Always returns the default FILE* based filesystem
+ */
+MsOleSysWrappers *
+ms_ole_get_default_fs (void)
+{
+	return &ms_ole_default_wrappers;
+}
+
+gint ms_ole_stream_read_copy   (MsOleStream *stream,
+				guint8 *ptr,
+				MsOlePos length)
+{
+	g_return_val_if_fail (stream != NULL, -1);
+	return stream->read_copy(stream, ptr, length);
+}
+
+guint8 * ms_ole_stream_read_ptr     (MsOleStream *stream,
+				     MsOlePos length)
+{
+	g_return_val_if_fail(stream != NULL, NULL);
+	return stream->read_ptr(stream, length);
+}
+
+MsOleSPos ms_ole_stream_lseek        (MsOleStream *stream,
+				      MsOleSPos bytes,
+				      MsOleSeek type)
+{
+	g_return_val_if_fail (stream != NULL, -1);
+	return stream->lseek (stream, bytes, type);
+}
+
+MsOlePos  ms_ole_stream_tell         (MsOleStream *stream)
+{
+	g_return_val_if_fail (stream != NULL, -1);
+	return stream->tell(stream);
+}
+
+MsOlePos  ms_ole_stream_write       (MsOleStream *stream,
+				     guint8 *ptr,
+				     MsOlePos length)
+{
+	g_return_val_if_fail (stream != NULL, -1);
+	return stream->write(stream, ptr, length);
+}
+
 /*
  * A global variable to enable calles to check_stream,
  * applications should optionally enable due to the performance penalty.
@@ -331,8 +395,8 @@ write_cache_block (MsOle *f, BBBlkAttr *attr)
 	g_return_if_fail (attr->data);
 
 	offset = (attr->blk+1)*BB_BLOCK_SIZE;
-	if (f->syswrap->lseek (f->file_des, offset, SEEK_SET)==(off_t)-1 ||
-	    f->syswrap->write (f->file_des, attr->data, BB_BLOCK_SIZE) == -1)
+	if (f->syswrap->lseek (f->file_des, offset, SEEK_SET, f->syswrap->closure) == (off_t)-1 ||
+	    f->syswrap->write (f->file_des, attr->data, BB_BLOCK_SIZE, f->syswrap->closure) == -1)
 		g_warning ("Fatal error writing block %d at %d\n", attr->blk, offset);
 #if OLE_DEBUG > 2
 	g_print ("Writing cache block %d to offset %d\n",
@@ -375,7 +439,7 @@ get_block_ptr (MsOle *f, BLP b, gboolean forwrite)
 		        else if (tmp->usage < min->usage)
 				min = tmp;
 		}
-		tmp->usage = (guint32)tmp->usage*0.707;
+		tmp->usage = (guint32)(tmp->usage*0.707);
 	}
 	if (blks < MAX_CACHED_BLOCKS)
 		min = 0;
@@ -395,8 +459,8 @@ get_block_ptr (MsOle *f, BLP b, gboolean forwrite)
 		attr->data = g_new (guint8, BB_BLOCK_SIZE);
 
 	offset = (b+1)*BB_BLOCK_SIZE;
-	f->syswrap->lseek (f->file_des, offset, SEEK_SET);
-	f->syswrap->read (f->file_des, attr->data, BB_BLOCK_SIZE);
+	f->syswrap->lseek (f->file_des, offset, SEEK_SET, f->syswrap->closure);
+	f->syswrap->read (f->file_des, attr->data, BB_BLOCK_SIZE, f->syswrap->closure);
 	attr->usage = 1;
 	attr->dirty = forwrite;
 
@@ -586,7 +650,7 @@ dump_allocation (MsOle *f)
 	int lp;
 	char *blktype;
 
-	for (lp = 0;lp<f->bb->len;lp++) {
+	for (lp = 0;lp<(int)f->bb->len;lp++) {
 		characterise_block (f, lp, &blktype);
 		g_print ("Block %d -> block %d ( '%s' )\n", lp,
 			 g_array_index (f->bb, BLP, lp),
@@ -785,7 +849,7 @@ read_bb (MsOle *f)
 static void
 remap_file (MsOle *f, guint blocks)
 {
-	int file;
+	MsOleHandleType file;
 	guint8 *newptr, zero = 0;
 	guint32 filesize;
 	guint32 oldlen;
@@ -796,16 +860,16 @@ remap_file (MsOle *f, guint blocks)
 	g_assert (f);
 	file = f->file_des;
 
-	g_assert (f->syswrap->munmap (f->mem, f->length) != -1);
+	g_assert (f->syswrap->munmap (f->mem, f->length, f->syswrap->closure) != -1);
 
 	/* Extend that file by blocks */
-	if (f->syswrap->getfilesize (file, &filesize)) {
+	if (f->syswrap->getfilesize (file, &filesize, f->syswrap->closure)) {
 		g_warning ("Serious error extending file\n");
 		f->mem = 0;
 		return;
 	}
 
-	if (f->syswrap->lseek (file, 0, SEEK_END) == (off_t)-1) {
+	if (f->syswrap->lseek (file, 0, SEEK_END, f->syswrap->closure) == (off_t)-1) {
 		g_warning ("Serious error extending file\n");
 		f->mem = 0;
 		return;
@@ -813,14 +877,15 @@ remap_file (MsOle *f, guint blocks)
 
 	for (icount = 0; icount < blocks; icount++) {
 		if (f->syswrap->write (file, zeroblock, BB_BLOCK_SIZE -
-				       ((icount == blocks - 1) ? 1 : 0))
+				       ((icount == blocks - 1) ? 1 : 0),
+				       f->syswrap->closure)
 		    == -1) {
 			g_warning ("Serious error extending file\n");
 			f->mem = 0;
 			return;
 		}
 	}
-	if (f->syswrap->write (file, &zero, 1) == -1) {
+	if (f->syswrap->write (file, &zero, 1, f->syswrap->closure) == -1) {
 		g_warning ("Serious error extending file\n");
 		f->mem = 0;
 		return;
@@ -828,7 +893,7 @@ remap_file (MsOle *f, guint blocks)
 
 	oldlen = filesize;
 
-	if (f->syswrap->getfilesize (file, &(f->length)))
+	if (f->syswrap->getfilesize (file, &(f->length), f->syswrap->closure))
 		g_warning ("couldn't get the size of the file\n");
 
 	g_assert (f->length == BB_BLOCK_SIZE*blocks + oldlen);
@@ -837,7 +902,7 @@ remap_file (MsOle *f, guint blocks)
 		g_warning ("file %d non-integer number of blocks\n", f->length);
 
 	newptr = f->syswrap->mmap (f->mem, f->length, PROT_READ|PROT_WRITE,
-				   MAP_SHARED, file, 0);
+				   MAP_SHARED, file, 0, f->syswrap->closure);
 #if OLE_DEBUG > 0
 	if (newptr != f->mem)
 		g_print ("Memory map moved from %p to %p\n",
@@ -1321,7 +1386,6 @@ destroy_pps (GList *l)
 	g_list_free (l);
 }
 
-
 static int
 write_pps (MsOle *f)
 {
@@ -1331,7 +1395,7 @@ write_pps (MsOle *f)
 	BLP last = END_OF_CHAIN;
 
 	/* Build the root chain */
-	for (lp = 0;lp<(f->num_pps+(BB_BLOCK_SIZE/PPS_BLOCK_SIZE)-1)/(BB_BLOCK_SIZE/PPS_BLOCK_SIZE);lp++) {
+	for (lp = 0;lp<(int)(f->num_pps+(BB_BLOCK_SIZE/PPS_BLOCK_SIZE)-1)/(BB_BLOCK_SIZE/PPS_BLOCK_SIZE);lp++) {
 		last  = blk;
 		blk   = next_free_bb (f);
 		g_assert (g_array_index (f->bb, BLP, blk) == UNUSED_BLOCK);
@@ -1408,7 +1472,7 @@ read_sb (MsOle *f)
 		if (ptr == UNUSED_BLOCK ||
 		    ptr == SPECIAL_BLOCK) {
 			g_warning ("Corrupt file descriptor: serious error, "
-				 "invalid block in chain\n");
+				   "invalid block in chain\n");
 			g_array_free (f->sb, TRUE);
 			f->sb = 0;
 			return 0;
@@ -1610,7 +1674,7 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 {
 	int prot = PROT_READ | PROT_WRITE;
 	MsOle *f;
-	int file;
+	MsOleHandleType file;
 
 	if (!fs)
 		return MS_OLE_ERR_BADARG;
@@ -1622,28 +1686,26 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 	f = *fs = ms_ole_new ();
 	take_wrapper_functions (f, wrappers);
 
-	f->file_des = file = f->syswrap->open2 (name, O_RDWR);
+	f->file_des = file = f->syswrap->open2 (name, O_RDWR, f->syswrap->closure);
 	f->ref_count = 0;
 	f->mode = 'w';
 
-	if (file == -1) {
-		f->file_des = file = f->syswrap->open2 (name, O_RDONLY);
+	if (file == BAD_MSOLE_HANDLE) {
+		f->file_des = file = f->syswrap->open2 (name, O_RDONLY, f->syswrap->closure);
 		f->mode = 'r';
-#ifdef HAVE_MMAP
 		prot &= ~PROT_WRITE;
-#endif
 	}
 
-	if ((file == -1) || !(f->syswrap->isregfile (file))) {
+	if ((file == BAD_MSOLE_HANDLE) || !(f->syswrap->isregfile (file, f->syswrap->closure))) {
 		g_warning ("No such file '%s'\n", name);
 		g_free (f) ;
 		*fs = NULL;
 		return MS_OLE_ERR_EXIST;
 	}
 
-	if (f->syswrap->getfilesize (file, &(f->length) )) {
+	if (f->syswrap->getfilesize (file, &(f->length), f->syswrap->closure)) {
 		g_warning ("Couldn't get the size of file '%s'\n", name);
-		f->syswrap->close (file) ;
+		f->syswrap->close (file, f->syswrap->closure) ;
 		g_free (f);
 		*fs = NULL;
 		return MS_OLE_ERR_EXIST;
@@ -1653,7 +1715,7 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 #if OLE_DEBUG > 0
 		g_warning ("File '%s' too short\n", name);
 #endif
-		f->syswrap->close (file) ;
+		f->syswrap->close (file, f->syswrap->closure) ;
 		g_free (f) ;
 		*fs = NULL;
 		return MS_OLE_ERR_FORMAT;
@@ -1661,7 +1723,7 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 
 	if (try_mmap && f->syswrap->mmap) {
 		f->mem = f->syswrap->mmap (
-			0, f->length, prot, MAP_SHARED, file, 0);
+			0, f->length, prot, MAP_SHARED, file, 0, f->syswrap->closure);
 		if (!f->mem || (caddr_t)f->mem == (caddr_t)MAP_FAILED) {
 			g_warning ("I can't mmap that file, falling back to slower method");
 			f->mem = NULL;
@@ -1678,9 +1740,10 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 		f->mem = g_new (guint8, BB_BLOCK_SIZE);
 
 		if (!f->mem ||
-		    f->syswrap->read (file, f->mem, BB_BLOCK_SIZE == -1)) {
+		    f->syswrap->read (file, f->mem, (BB_BLOCK_SIZE == -1),
+				      f->syswrap->closure)) {
 			g_warning ("Error reading header\n");
-			f->syswrap->close (file) ;
+			f->syswrap->close (file, f->syswrap->closure) ;
 			g_free (f);
 			*fs = NULL;
 			return MS_OLE_ERR_EXIST;
@@ -1745,7 +1808,8 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 		   MsOleSysWrappers *wrappers)
 {
 	MsOle *f;
-	int file, zero = 0;
+	MsOleHandleType file;
+	int zero = 0;
 	int init_blocks = 1, lp;
 
 	if (!fs)
@@ -1754,9 +1818,10 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 	f = *fs = ms_ole_new ();
 	take_wrapper_functions (f, wrappers);
 	if ((file = f->syswrap->open3 (name,
-					  O_RDWR|O_CREAT|O_TRUNC,
-					  S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP))
-	    == -1) {
+				       O_RDWR|O_CREAT|O_TRUNC,
+				       S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP,
+				       f->syswrap->closure))
+	    == BAD_MSOLE_HANDLE) {
 		g_warning ("Can't create file '%s'\n", name);
 		g_free (f);
 		*fs = NULL;
@@ -1764,8 +1829,8 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 	}
 
 	if ((f->syswrap->lseek (file, BB_BLOCK_SIZE * init_blocks - 1,
-		    SEEK_SET) == (off_t)-1) ||
-	    (f->syswrap->write (file, &zero, 1) == -1)) {
+		    SEEK_SET, f->syswrap->closure) == (off_t)-1) ||
+	    (f->syswrap->write (file, &zero, 1, f->syswrap->closure) == -1)) {
 		g_warning ("Serious error extending file to %d bytes\n",
 			   BB_BLOCK_SIZE*init_blocks);
 		g_free (f);
@@ -1776,7 +1841,7 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 	f->ref_count = 0;
 	f->file_des  = file;
 	f->mode      = 'w';
-	if (f->syswrap->getfilesize (file, &(f->length))) {
+	if (f->syswrap->getfilesize (file, &(f->length), f->syswrap->closure)) {
 		g_warning ("Warning couldn't get the size of the file '%s'\n",
 			   name);
 	}
@@ -1788,7 +1853,7 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 		if (f->syswrap->mmap)
 			f->mem = f->syswrap->mmap (
 				0, f->length, PROT_READ | PROT_WRITE,
-				MAP_SHARED, file, 0);
+				MAP_SHARED, file, 0, f->syswrap->closure);
 		else
 			f->mem = NULL;
 
@@ -1898,9 +1963,11 @@ ms_ole_destroy (MsOle **ptr)
 			}
 
 			if (f->dirty) {
-				f->syswrap->lseek (f->file_des, 0, SEEK_SET);
+				f->syswrap->lseek (f->file_des, 0, SEEK_SET,
+						   f->syswrap->closure);
 				f->syswrap->write (f->file_des, f->mem,
-						   BB_BLOCK_SIZE);
+						   BB_BLOCK_SIZE,
+						   f->syswrap->closure);
 			}
 			g_free (f->mem);
 			f->mem = NULL;
@@ -1909,7 +1976,7 @@ ms_ole_destroy (MsOle **ptr)
 		destroy_pps (f->pps);
 		f->pps = NULL;
 
-		f->syswrap->close (f->file_des);
+		f->syswrap->close (f->file_des, f->syswrap->closure);
 		g_free (f);
 
 #if OLE_DEBUG > 0
@@ -1919,7 +1986,6 @@ ms_ole_destroy (MsOle **ptr)
 	*ptr = NULL;
 }
 
-
 /**
  * ms_ole_dump:
  * @ptr: memory area to be dumped.
@@ -2105,13 +2171,13 @@ ms_ole_lseek (MsOleStream *s, MsOleSPos bytes, MsOleSeek type)
 	g_return_val_if_fail (s, -1);
 
 	if (type == MsOleSeekSet)
-		newpos = bytes;
+		newpos  = bytes;
 	else if (type == MsOleSeekCur)
 		newpos = s->position + bytes;
 	else
 		newpos = s->size + bytes;
 
-	if (newpos > s->size || newpos < 0) {
+	if (newpos > (MsOleSPos) s->size || newpos < 0) {
 		g_warning ("Invalid seek");
 		return -1;
 	}
@@ -2129,33 +2195,33 @@ ms_ole_read_ptr_bb (MsOleStream *s, MsOlePos length)
 {
 	int blklen;
 	guint8 *ans;
-	guint32 len = length;
-	int blockidx = s->position / BB_BLOCK_SIZE;
+	guint32 len  = length;
+	int blockidx = s->position/BB_BLOCK_SIZE;
 
 	g_return_val_if_fail (s, NULL);
 
-	if (!s->blocks || blockidx >= s->blocks->len) {
+	if (!s->blocks || blockidx >= (int) s->blocks->len) {
 		g_warning ("Reading from NULL file\n");
 		return NULL;
 	}
 
 	blklen = BB_BLOCK_SIZE - s->position % BB_BLOCK_SIZE;
 
-	if (len > blklen && !s->file->ole_mmap)
+	if (len > (guint32) blklen && !s->file->ole_mmap)
 		return NULL;
 
-	while (len > blklen) {
+	while (len > (guint32) blklen) {
 		len -= blklen;
 		blklen = BB_BLOCK_SIZE;
-		if (blockidx >= (s->blocks->len - 1)
+		if (blockidx >= (int) (s->blocks->len - 1)
 		    || (ms_array_index (s->blocks, BLP, blockidx)
-			!= blockidx + 1))
+			!= (BLP) (blockidx + 1) ))
 			return NULL;
 		blockidx++;
 	}
 	/* Straight map, simply return a pointer */
 	ans = BB_R_PTR (s->file, ms_array_index (s->blocks, BLP,
-						 s->position / BB_BLOCK_SIZE))
+						 s->position/BB_BLOCK_SIZE))
 		+ s->position % BB_BLOCK_SIZE;
 	ms_ole_lseek (s, length, MsOleSeekCur);
 
@@ -2175,34 +2241,34 @@ ms_ole_read_ptr_sb (MsOleStream *s, MsOlePos length)
 {
 	int blklen;
 	guint8 *ans;
-	guint32 len = length;
+	guint32 len  = length;
 	int blockidx = s->position / SB_BLOCK_SIZE;
 
 	g_return_val_if_fail (s, NULL);
 
-	if (!s->blocks || blockidx >= s->blocks->len) {
+	if (!s->blocks || blockidx >= (int) s->blocks->len) {
 		g_warning ("Reading from NULL file\n");
 		return NULL;
 	}
 
 	blklen = SB_BLOCK_SIZE - s->position % SB_BLOCK_SIZE;
 
-	if (len > blklen && !s->file->ole_mmap)
+	if (len > (guint32) blklen && !s->file->ole_mmap)
 		return NULL;
 
-	while (len > blklen) {
+	while (len > (guint32) blklen) {
 		len -= blklen;
 		blklen = SB_BLOCK_SIZE;
-		if (blockidx >= (s->blocks->len - 1)
+		if (blockidx >= (int) (s->blocks->len - 1)
 		    || (ms_array_index (s->blocks, BLP, blockidx)
-			!= blockidx + 1))
+			!= (BLP) (blockidx + 1) ))
 			return NULL;
 		blockidx++;
 	}
 	/* Straight map, simply return a pointer */
 	ans = GET_SB_R_PTR (s->file, ms_array_index (s->blocks, BLP,
-						     s->position / SB_BLOCK_SIZE))
-		+ s->position%SB_BLOCK_SIZE;
+						     s->position/SB_BLOCK_SIZE))
+		+ s->position % SB_BLOCK_SIZE;
 	ms_ole_lseek (s, length, MsOleSeekCur);
 
 	if (libole2_debug)
@@ -2236,18 +2302,18 @@ ms_ole_read_copy_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 	{
 		BLP block;
 		int cpylen = BB_BLOCK_SIZE - offset;
-		if (cpylen > length)
+		if (cpylen > (int) length)
 			cpylen = length;
 
 		if (s->position + cpylen > s->size
-		    || blkidx == s->blocks->len) {
+		    || blkidx == (int) s->blocks->len) {
 #if OLE_DEBUG > 0
 			g_print ("Trying 2 to read beyond end of stream %d+%d %d\n",
 				s->position, cpylen, s->size);
 #endif
 			return 0;
 		}
-		g_assert (blkidx < s->blocks->len);
+		g_assert (blkidx < (int) s->blocks->len);
 		block = ms_array_index (s->blocks, BLP, blkidx);
 		src = BB_R_PTR (s->file, block) + offset;
 
@@ -2258,7 +2324,7 @@ ms_ole_read_copy_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 		offset = 0;
 
 		blkidx++;
-		s->position += cpylen;
+		s->position+=cpylen;
 	}
 
 	if (libole2_debug)
@@ -2292,17 +2358,17 @@ ms_ole_read_copy_sb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 	{
 		int cpylen = SB_BLOCK_SIZE - offset;
 		BLP block;
-		if (cpylen>length)
+		if (cpylen>(int)length)
 			cpylen = length;
 		if (s->position + cpylen > s->size
-		    || blkidx == s->blocks->len) {
+		    || blkidx == (int) s->blocks->len) {
 #if OLE_DEBUG > 0
 			g_print ("Trying 3 to read beyond end of stream %d+%d %d\n",
 				s->position, cpylen, s->size);
 #endif
 			return 0;
 		}
-		g_assert (blkidx < s->blocks->len);
+		g_assert (blkidx < (int) s->blocks->len);
 		block = ms_array_index (s->blocks, BLP, blkidx);
 		src = GET_SB_R_PTR (s->file, block) + offset;
 
@@ -2365,7 +2431,7 @@ ms_ole_append_block (MsOleStream *s)
 		{
 			int lp;
 			g_assert (g_array_index (s->file->bb, BLP, block) == UNUSED_BLOCK);
-			for (lp = 0;lp<s->blocks->len;lp++)
+			for (lp=0;lp<s->blocks->len;lp++)
 				g_assert (g_array_index (s->blocks, BLP, lp) != block);
 		}
 #endif
@@ -2403,7 +2469,7 @@ ms_ole_write_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 	while (bytes > 0) {
 		BLP block;
 		int cpylen = BB_BLOCK_SIZE - offset;
-		if (cpylen > bytes)
+		if (cpylen > (int) bytes)
 			cpylen = bytes;
 
 		if (!s->blocks || blkidx >= s->blocks->len)
@@ -2414,7 +2480,7 @@ ms_ole_write_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 
 		dest = BB_W_PTR(s->file, block) + offset;
 
-#if OLE_DEBUG > 2
+#if OLE_DEBUG > 1
 		g_print ("Copy %d bytes to block %d\n", cpylen, block);
 #endif
 		memcpy (dest, ptr, cpylen);
@@ -2453,7 +2519,7 @@ ms_ole_write_sb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 		BLP block;
 		int cpylen = SB_BLOCK_SIZE - offset;
 
-		if (cpylen > bytes)
+		if (cpylen > (int) bytes)
 			cpylen = bytes;
 
 		if (!s->blocks || blkidx >= s->blocks->len)
@@ -2928,9 +2994,9 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
 		else
 			s->blocks = NULL;
 
-		s->type = MsOleSmallBlock;
+		s->type   = MsOleSmallBlock;
 
-		for (lp = 0; !panic && (lp < (s->size + SB_BLOCK_SIZE - 1) / SB_BLOCK_SIZE); lp++) {
+		for (lp = 0; !panic & (lp < (int) (s->size + SB_BLOCK_SIZE - 1) / SB_BLOCK_SIZE); lp++) {
 			g_array_append_val (s->blocks, b);
 #if OLE_DEBUG > 0
 			g_print ("Block %d\n", b);
@@ -2976,34 +3042,6 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
 }
 
 
-/**
- * ms_ole_stream_duplicate:
- * @stream_copy: stream object copy.
- * @stream: stream object to be duplicated.
- *
- * Duplicates the stream object @stream.
- *
- * Return value: a #MsOleErr code.
- **/
-MsOleErr
-ms_ole_stream_duplicate (MsOleStream ** const s, const MsOleStream * const stream)
-{
-	if (!s || !stream)
-		return MS_OLE_ERR_BADARG;
-
-	/* Lets face it having two pointers to the same file is a nightmare anyway :-) */
-	g_warning ("Do NOT use this function, it is unsafe with the blocks array");
-
-	if (!(*s = g_new (MsOleStream, 1)))
-		return MS_OLE_ERR_MEM;
-
-	memcpy (*s, stream, sizeof (MsOleStream));
-	ms_ole_ref (stream->file);
-
-	return MS_OLE_ERR_OK;
-}
-
-
 /**
  * ms_ole_stream_close:
  * @stream: stream object to be closed.
